//
//  QUQRCodeScannerViewController.swift
//  qu_qr_code_scanner
//
//  Created by edy on 2021/7/15.
//

import AVFoundation
import UIKit

class QUQRCodeScannerViewController: UIViewController, AVCaptureMetadataOutputObjectsDelegate, UIAlertViewDelegate {
    var scanRectView = UIView()
    var device: AVCaptureDevice!
    var input: AVCaptureDeviceInput!
    var output: AVCaptureMetadataOutput!
    var session: AVCaptureSession!
    var preview: AVCaptureVideoPreviewLayer!

    var scanImg = UIImageView(image: QUImagesHelper.imageFor(name: "scan"))
    var choseImgBtn = UIButton()
    let maxWindowSize = max(UIScreen.main.bounds.size.width, UIScreen.main.bounds.size.height)
    let minWindowSize = min(UIScreen.main.bounds.size.width, UIScreen.main.bounds.size.height)
    lazy var scanSize = CGSize(width: minWindowSize * 3 / 4, height: minWindowSize * 3 / 4)

    var resultBlock: ((_ result: String) -> Void)?

    var qrString = ""
    var isEnLarge = true

    override func viewDidLoad() {
        super.viewDidLoad()

        initView()
        setScanner()

        orientationDidChangeNotification()
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        startAnimation()
    }

    // 摄像头捕获
    func metadataOutput(_: AVCaptureMetadataOutput,
                        didOutput metadataObjects: [AVMetadataObject],
                        from _: AVCaptureConnection)
    {
        var stringValue: String?
        if metadataObjects.count > 0 {
            let metadataObject = metadataObjects[0] as! AVMetadataMachineReadableCodeObject
            stringValue = metadataObject.stringValue

            if stringValue != nil {
                print("扫描到的二维码：", stringValue!)
                qrString = stringValue!
                if let b = resultBlock {
                    b(qrString)
                }
                session.stopRunning()
            }
        }
        session.stopRunning()
        dismiss(animated: true) {}
    }

    func startAnimation() {
        let animation = CABasicAnimation(keyPath: "transform.translation.y")
        animation.fromValue = 0
        animation.toValue = scanSize.height - 30
        animation.duration = 2
        animation.repeatCount = 99999
        scanImg.layer.add(animation, forKey: nil)
    }

    func initView() {
        let backImg = QUImagesHelper.imageFor(name: "back")
        let backBtn = UIBarButtonItem(image: backImg, style: .plain, target: self, action: #selector(backAction))
        backBtn.tintColor = UIColor.white
        navigationItem.setLeftBarButton(backBtn, animated: false)

        let img = QUImagesHelper.imageFor(name: "photo")
        choseImgBtn.setImage(img, for: .normal)
        choseImgBtn.addTarget(self, action: #selector(chooseImg), for: .touchUpInside)
        view.addSubview(choseImgBtn)

        view.isUserInteractionEnabled = true
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(tapGesture))
        tapGesture.numberOfTapsRequired = 2
        view.addGestureRecognizer(tapGesture)
    }

    @objc func tapGesture() {
        CATransaction.begin()
        CATransaction.setAnimationDuration(0.5)
        if isEnLarge {
            preview.transform = CATransform3DMakeScale(3, 3, 3)
        } else {
            preview.transform = CATransform3DMakeScale(1, 1, 1)
        }
        CATransaction.commit()
        isEnLarge = !isEnLarge
    }

    @objc
    private func backAction() {
        navigationController?.dismiss(animated: true, completion: nil)
    }

    @objc func chooseImg() {
        let pick = UIImagePickerController()
        pick.delegate = self
        pick.sourceType = .photoLibrary
        pick.modalPresentationStyle = .fullScreen
        present(pick, animated: true, completion: nil)
    }

    private func orientationDidChangeNotification() {
        let name = UIDevice.orientationDidChangeNotification
        NotificationCenter.default.addObserver(self, selector: #selector(orientationDidChangeHandle), name: name, object: nil)
    }

    @objc
    private func orientationDidChangeHandle() {
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
            self.updateScannerRect()
        }
    }

    private func setScanner() {
        do {
            // Do any additional setup after loading the view.
            device = AVCaptureDevice.default(for: AVMediaType.video)

            input = try AVCaptureDeviceInput(device: device)

            output = AVCaptureMetadataOutput()
            output.setMetadataObjectsDelegate(self, queue: DispatchQueue.main)

            session = AVCaptureSession()
            session.addInput(input)
            session.addOutput(output)

            output.metadataObjectTypes = [AVMetadataObject.ObjectType.qr]

            preview = AVCaptureVideoPreviewLayer(session: session)
            preview.videoGravity = AVLayerVideoGravity.resizeAspectFill
            view.layer.insertSublayer(preview, at: 0)

            // 添加中间的扫描特效活动范围试图
            view.addSubview(scanRectView)
            scanRectView.addSubview(scanImg)
            // 开始捕获
            session.startRunning()

        } catch _ {
            // 打印错误消息
            let alertController = UIAlertController(title: "提醒",
                                                    message: "请在iPhone的\"设置-隐私-相机\"选项中,允许本程序访问您的相机",
                                                    preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "确定", style: .cancel, handler: nil)
            alertController.addAction(cancelAction)
            present(alertController, animated: true, completion: nil)
        }
        updateScannerRect()
    }

    private func updateScannerRect() {
        startAnimation()
        let viewSize = view.frame.size
        preview.frame = view.bounds
        choseImgBtn.frame = CGRect(x: viewSize.width - 80, y: viewSize.height - 100, width: 60, height: 60)

        if maxWindowSize < 500 {
            session.sessionPreset = AVCaptureSession.Preset.vga640x480
        } else {
            session.sessionPreset = AVCaptureSession.Preset.high
        }

        scanRectView.frame = CGRect(x: 0, y: 0, width: minWindowSize,
                                    height: minWindowSize)
        scanRectView.center = CGPoint(x: UIScreen.main.bounds.midX,
                                      y: UIScreen.main.bounds.midY)
        scanImg.frame = CGRect(x: 0, y: 0, width: minWindowSize, height: 30)
        scanImg.contentMode = .scaleAspectFit

        let orientation = UIDevice.current.orientation
        if let connection = preview.connection {
            if connection.isVideoOrientationSupported,
               let videoOrientation = AVCaptureVideoOrientation(rawValue: orientation.rawValue)
            {
                connection.videoOrientation = videoOrientation
            }
        }
    }
}

extension QUQRCodeScannerViewController: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    func imagePickerController(_: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey: Any]) {
        dismiss(animated: true, completion: nil)
        var image: UIImage?

        image = info[.originalImage] as? UIImage

        let ciImage = CIImage(image: image!)!

        let context = CIContext(options: nil)
        let detector = CIDetector(ofType: CIDetectorTypeQRCode, context: context, options: [CIDetectorAccuracy: CIDetectorAccuracyHigh])

        let features = detector?.features(in: ciImage)
        for feature in features as! [CIQRCodeFeature] {
            qrString = feature.messageString ?? ""
            print("图片读取到的二维码", feature.messageString ?? "")
            if let b = resultBlock {
                b(qrString)
            }
            dismiss(animated: true) {}
        }
    }
}
